@@ -16,8 +16,9 @@ use std::{
 };
 
 use anyhow::{Context, Result};
-use config::style::ColourPalette;
 pub use config::Config;
+use config::style::Styles;
+use data::TemperatureType;
 pub(crate) use error::{OptionError, OptionResult};
 use hashbrown::{HashMap, HashSet};
 use indexmap::IndexSet;
@@ -27,13 +28,12 @@ use starship_battery::Manager;
 
 use self::{
     args::BottomArgs,
-    config::{layout::Row, IgnoreList, StringOrNum},
+    config::{IgnoreList, StringOrNum, layout::Row},
 };
 use crate::{
     app::{filter::Filter, layout_manager::*, *},
-    canvas::components::time_chart::LegendPosition,
+    canvas::components::time_graph::LegendPosition,
     constants::*,
-    data_collection::temperature::TemperatureType,
     utils::data_units::DataUnit,
     widgets::*,
 };
@@ -60,36 +60,67 @@ macro_rules! is_flag_enabled {
     };
 }
 
+/// The default config file sub-path.
+const DEFAULT_CONFIG_FILE_LOCATION: &str = "bottom/bottom.toml";
+
 /// Returns the config path to use. If `override_config_path` is specified, then
 /// we will use that. If not, then return the "default" config path, which is:
+///
 /// - If a path already exists at `<HOME>/bottom/bottom.toml`, then use that for
 ///   legacy reasons.
 /// - Otherwise, use `<SYSTEM_CONFIG_FOLDER>/bottom/bottom.toml`.
 ///
 /// For more details on this, see [dirs](https://docs.rs/dirs/latest/dirs/fn.config_dir.html)'
 /// documentation.
+///
+/// XXX: For macOS, we additionally will manually check `$XDG_CONFIG_HOME` as well first
+/// before falling back to `dirs`.
 fn get_config_path(override_config_path: Option<&Path>) -> Option<PathBuf> {
-    const DEFAULT_CONFIG_FILE_PATH: &str = "bottom/bottom.toml";
-
     if let Some(conf_loc) = override_config_path {
         return Some(conf_loc.to_path_buf());
     } else if let Some(home_path) = dirs::home_dir() {
         let mut old_home_path = home_path;
         old_home_path.push(".config/");
-        old_home_path.push(DEFAULT_CONFIG_FILE_PATH);
-        if old_home_path.exists() {
-            // We used to create it at `<HOME>/DEFAULT_CONFIG_FILE_PATH`, but changed it
-            // to be more correct later. However, for legacy reasons, if it already exists,
-            // use the old one.
-            return Some(old_home_path);
+        old_home_path.push(DEFAULT_CONFIG_FILE_LOCATION);
+        if let Ok(res) = old_home_path.try_exists() {
+            if res {
+                // We used to create it at `<HOME>/DEFAULT_CONFIG_FILE_PATH`, but changed it
+                // to be more correct later. However, for legacy reasons, if it already exists,
+                // use the old one.
+                return Some(old_home_path);
+            }
         }
     }
 
-    // Otherwise, return the "correct" path based on the config dir.
-    dirs::config_dir().map(|mut path| {
-        path.push(DEFAULT_CONFIG_FILE_PATH);
+    let config_path = dirs::config_dir().map(|mut path| {
+        path.push(DEFAULT_CONFIG_FILE_LOCATION);
         path
-    })
+    });
+
+    if cfg!(target_os = "macos") {
+        if let Ok(xdg_config_path) = std::env::var("XDG_CONFIG_HOME") {
+            if !xdg_config_path.is_empty() {
+                // If XDG_CONFIG_HOME exists and is non-empty, _but_ we previously used the Library-based path
+                // for a config and it exists, then use that instead for backwards-compatibility.
+                if let Some(old_macos_path) = &config_path {
+                    if let Ok(res) = old_macos_path.try_exists() {
+                        if res {
+                            return config_path;
+                        }
+                    }
+                }
+
+                // Otherwise, try and use the XDG_CONFIG_HOME-based path.
+                let mut cfg_path = PathBuf::new();
+                cfg_path.push(xdg_config_path);
+                cfg_path.push(DEFAULT_CONFIG_FILE_LOCATION);
+
+                return Some(cfg_path);
+            }
+        }
+    }
+
+    config_path
 }
 
 fn create_config_at_path(path: &Path) -> anyhow::Result<Config> {
@@ -113,7 +144,7 @@ fn create_config_at_path(path: &Path) -> anyhow::Result<Config> {
 /// - If the user does NOT pass in a path explicitly, then just show a warning,
 ///   but continue. This is in case they do not want to write a default config file at
 ///   the XDG locations, for example.
-pub fn get_or_create_config(config_path: Option<&Path>) -> anyhow::Result<Config> {
+pub(crate) fn get_or_create_config(config_path: Option<&Path>) -> anyhow::Result<Config> {
     let adjusted_config_path = get_config_path(config_path);
 
     match &adjusted_config_path {
@@ -164,9 +195,8 @@ pub fn get_or_create_config(config_path: Option<&Path>) -> anyhow::Result<Config
     }
 }
 
-pub(crate) fn init_app(
-    args: BottomArgs, config: Config,
-) -> Result<(App, BottomLayout, ColourPalette)> {
+/// Initialize the app.
+pub(crate) fn init_app(args: BottomArgs, config: Config) -> Result<(App, BottomLayout, Styles)> {
     use BottomWidgetType::*;
 
     // Since everything takes a reference, but we want to take ownership here to
@@ -174,7 +204,7 @@ pub(crate) fn init_app(
     let args = &args;
     let config = &config;
 
-    let styling = ColourPalette::new(args, config)?;
+    let styling = Styles::new(args, config)?;
 
     let (widget_layout, default_widget_id, default_widget_type_option) =
         get_widget_layout(args, config)
@@ -194,8 +224,10 @@ pub(crate) fn init_app(
     let is_use_regex = is_flag_enabled!(regex, args.process, config);
     let is_default_tree = is_flag_enabled!(tree, args.process, config);
     let is_default_command = is_flag_enabled!(process_command, args.process, config);
+    #[cfg(any(target_os = "linux", target_os = "macos", target_os = "freebsd"))]
     let is_advanced_kill = !(is_flag_enabled!(disable_advanced_kill, args.process, config));
     let process_memory_as_value = is_flag_enabled!(process_memory_as_value, args.process, config);
+    let is_default_tree_collapsed = is_flag_enabled!(tree_collapse, args.process, config);
 
     // For CPU
     let default_cpu_selection = get_default_cpu_selection(args, config);
@@ -230,6 +262,7 @@ pub(crate) fn init_app(
             if cfg.columns.is_empty() {
                 None
             } else {
+                // TODO: Should we be using an indexmap? Or maybe allow dupes.
                 Some(IndexSet::from_iter(
                     cfg.columns.iter().map(ProcWidgetColumn::from),
                 ))
@@ -265,6 +298,7 @@ pub(crate) fn init_app(
             args.general,
             config
         ),
+        #[cfg(any(target_os = "linux", target_os = "macos", target_os = "freebsd"))]
         is_advanced_kill,
         memory_legend_position,
         network_legend_position,
@@ -273,6 +307,7 @@ pub(crate) fn init_app(
         network_use_binary_prefix,
         retention_ms,
         dedicated_average_row: get_dedicated_avg_row(config),
+        default_tree_collapse: is_default_tree_collapsed,
     };
 
     let table_config = ProcTableConfig {
@@ -350,9 +385,7 @@ pub(crate) fn init_app(
                             let mode = if is_grouped {
                                 ProcWidgetMode::Grouped
                             } else if is_default_tree {
-                                ProcWidgetMode::Tree {
-                                    collapsed_pids: Default::default(),
-                                }
+                                ProcWidgetMode::Tree(TreeCollapsed::new(is_default_tree_collapsed))
                             } else {
                                 ProcWidgetMode::Normal
                             };
@@ -371,7 +404,11 @@ pub(crate) fn init_app(
                         Disk => {
                             disk_state_map.insert(
                                 widget.widget_id,
-                                DiskTableWidget::new(&app_config_fields, &styling),
+                                DiskTableWidget::new(
+                                    &app_config_fields,
+                                    &styling,
+                                    config.disk.as_ref().map(|cfg| cfg.columns.as_slice()),
+                                ),
                             );
                         }
                         Temp => {
@@ -396,7 +433,6 @@ pub(crate) fn init_app(
             Proc | Disk | Temp => BasicTableWidgetState {
                 currently_displayed_widget_type: initial_widget_type,
                 currently_displayed_widget_id: initial_widget_id,
-                widget_id: 100,
                 left_tlc: None,
                 left_brc: None,
                 right_tlc: None,
@@ -405,7 +441,6 @@ pub(crate) fn init_app(
             _ => BasicTableWidgetState {
                 currently_displayed_widget_type: Proc,
                 currently_displayed_widget_id: DEFAULT_WIDGET_ID,
-                widget_id: 100,
                 left_tlc: None,
                 left_brc: None,
                 right_tlc: None,
@@ -460,7 +495,7 @@ pub(crate) fn init_app(
         proc_state: ProcState::init(proc_state_map),
         temp_state: TempState::init(temp_state_map),
         disk_state: DiskState::init(disk_state_map),
-        battery_state: BatteryState::init(battery_state_map),
+        battery_state: AppBatteryState::init(battery_state_map),
         basic_table_widget_state,
     };
 
@@ -488,7 +523,7 @@ pub(crate) fn init_app(
     ))
 }
 
-pub fn get_widget_layout(
+fn get_widget_layout(
     args: &BottomArgs, config: &Config,
 ) -> OptionResult<(BottomLayout, u64, Option<BottomWidgetType>)> {
     let cpu_left_legend = is_flag_enabled!(cpu_left_legend, args.cpu, config);
@@ -637,8 +672,11 @@ macro_rules! parse_ms_option {
     }};
 }
 
+/// How fast the screen refreshes
 #[inline]
 fn get_update_rate(args: &BottomArgs, config: &Config) -> OptionResult<u64> {
+    const DEFAULT_REFRESH_RATE_IN_MILLISECONDS: u64 = 1000;
+
     parse_ms_option!(
         &args.general.rate,
         config.flags.as_ref().and_then(|flags| flags.rate.as_ref()),
@@ -702,6 +740,8 @@ fn get_dedicated_avg_row(config: &Config) -> bool {
 fn get_default_time_value(
     args: &BottomArgs, config: &Config, retention_ms: u64,
 ) -> OptionResult<u64> {
+    const DEFAULT_TIME_MILLISECONDS: u64 = 60 * 1000; // Defaults to 1 min.
+
     parse_ms_option!(
         &args.general.default_time_value,
         config
@@ -717,6 +757,8 @@ fn get_default_time_value(
 
 #[inline]
 fn get_time_interval(args: &BottomArgs, config: &Config, retention_ms: u64) -> OptionResult<u64> {
+    const TIME_CHANGE_MILLISECONDS: u64 = 15 * 1000; // How much to increment each time
+
     parse_ms_option!(
         &args.general.time_delta,
         config
@@ -780,61 +822,67 @@ fn get_default_widget_and_count(
     }
 }
 
-#[allow(unused_variables)]
+#[cfg(feature = "battery")]
 fn get_use_battery(args: &BottomArgs, config: &Config) -> bool {
-    #[cfg(feature = "battery")]
-    {
-        // TODO: Move this so it's dynamic in the app itself and automatically hide if
-        // there are no batteries?
-        if let Ok(battery_manager) = Manager::new() {
-            if let Ok(batteries) = battery_manager.batteries() {
-                if batteries.count() == 0 {
-                    return false;
-                }
+    // TODO: Move this so it's dynamic in the app itself and automatically hide if
+    // there are no batteries?
+    if let Ok(battery_manager) = Manager::new() {
+        if let Ok(batteries) = battery_manager.batteries() {
+            if batteries.count() == 0 {
+                return false;
             }
         }
+    }
 
-        if args.battery.battery {
-            return true;
-        } else if let Some(flags) = &config.flags {
-            if let Some(battery) = flags.battery {
-                return battery;
-            }
+    if args.battery.battery {
+        return true;
+    } else if let Some(flags) = &config.flags {
+        if let Some(battery) = flags.battery {
+            return battery;
         }
     }
 
     false
 }
 
-#[allow(unused_variables)]
+#[cfg(not(feature = "battery"))]
+fn get_use_battery(_args: &BottomArgs, _config: &Config) -> bool {
+    false
+}
+
+#[cfg(feature = "gpu")]
 fn get_enable_gpu(args: &BottomArgs, config: &Config) -> bool {
-    #[cfg(feature = "gpu")]
-    {
-        if args.gpu.enable_gpu {
-            return true;
-        } else if let Some(flags) = &config.flags {
-            if let Some(enable_gpu) = flags.enable_gpu {
-                return enable_gpu;
-            }
+    if args.gpu.disable_gpu {
+        return false;
+    }
+
+    !config
+        .flags
+        .as_ref()
+        .and_then(|f| f.disable_gpu)
+        .unwrap_or(false)
+}
+
+#[cfg(not(feature = "gpu"))]
+fn get_enable_gpu(_: &BottomArgs, _: &Config) -> bool {
+    false
+}
+
+#[cfg(not(target_os = "windows"))]
+fn get_enable_cache_memory(args: &BottomArgs, config: &Config) -> bool {
+    if args.memory.enable_cache_memory {
+        return true;
+    } else if let Some(flags) = &config.flags {
+        if let Some(enable_cache_memory) = flags.enable_cache_memory {
+            return enable_cache_memory;
         }
     }
 
     false
 }
 
-#[allow(unused_variables)]
-fn get_enable_cache_memory(args: &BottomArgs, config: &Config) -> bool {
-    #[cfg(not(target_os = "windows"))]
-    {
-        if args.memory.enable_cache_memory {
-            return true;
-        } else if let Some(flags) = &config.flags {
-            if let Some(enable_cache_memory) = flags.enable_cache_memory {
-                return enable_cache_memory;
-            }
-        }
-    }
-
+#[cfg(target_os = "windows")]
+fn get_enable_cache_memory(_args: &BottomArgs, _config: &Config) -> bool {
     false
 }
 
@@ -868,10 +916,7 @@ fn get_ignore_list(ignore_list: &Option<IgnoreList>) -> OptionResult<Option<Filt
 
         let list = list.map_err(|err| OptionError::config(err.to_string()))?;
 
-        Ok(Some(Filter {
-            list,
-            is_list_ignored: ignore_list.is_list_ignored,
-        }))
+        Ok(Some(Filter::new(ignore_list.is_list_ignored, list)))
     } else {
         Ok(None)
     }
@@ -927,11 +972,14 @@ fn get_network_legend_position(
     let result = if let Some(s) = &args.network.network_legend {
         match s.to_ascii_lowercase().trim() {
             "none" => None,
-            position => Some(parse_config_value!(position.parse(), "network_legend")?),
+            position => Some(parse_arg_value!(position.parse(), "network_legend")?),
         }
     } else if let Some(flags) = &config.flags {
-        if let Some(legend) = &flags.network_legend {
-            Some(parse_arg_value!(legend.parse(), "network_legend")?)
+        if let Some(s) = &flags.network_legend {
+            match s.to_ascii_lowercase().trim() {
+                "none" => None,
+                position => Some(parse_config_value!(position.parse(), "network_legend")?),
+            }
         } else {
             Some(LegendPosition::default())
         }
@@ -948,11 +996,14 @@ fn get_memory_legend_position(
     let result = if let Some(s) = &args.memory.memory_legend {
         match s.to_ascii_lowercase().trim() {
             "none" => None,
-            position => Some(parse_config_value!(position.parse(), "memory_legend")?),
+            position => Some(parse_arg_value!(position.parse(), "memory_legend")?),
         }
     } else if let Some(flags) = &config.flags {
-        if let Some(legend) = &flags.memory_legend {
-            Some(parse_arg_value!(legend.parse(), "memory_legend")?)
+        if let Some(s) = &flags.memory_legend {
+            match s.to_ascii_lowercase().trim() {
+                "none" => None,
+                position => Some(parse_config_value!(position.parse(), "memory_legend")?),
+            }
         } else {
             Some(LegendPosition::default())
         }
@@ -967,7 +1018,7 @@ fn get_memory_legend_position(
 mod test {
     use clap::Parser;
 
-    use super::{get_time_interval, Config};
+    use super::{Config, get_time_interval};
     use crate::{
         app::App,
         args::BottomArgs,
@@ -1184,4 +1235,50 @@ mod test {
             }
         }
     }
+
+    /// This one has slightly more complex behaviour due to `dirs` not respecting XDG on macOS, so we manually
+    /// handle it. However, to ensure backwards-compatibility, we also have to do some special cases.
+    #[cfg(target_os = "macos")]
+    #[test]
+    fn test_get_config_path_macos() {
+        use std::path::PathBuf;
+
+        use super::{DEFAULT_CONFIG_FILE_LOCATION, get_config_path};
+
+        // Case three: no previous config, no XDG var.
+        // SAFETY: This is fine, this is just a test, and no other test affects env vars.
+        unsafe {
+            std::env::remove_var("XDG_CONFIG_HOME");
+        }
+
+        let case_1 = dirs::config_dir()
+            .map(|mut path| {
+                path.push(DEFAULT_CONFIG_FILE_LOCATION);
+                path
+            })
+            .unwrap();
+
+        // Skip this test if the file already exists.
+        if !case_1.exists() {
+            assert_eq!(get_config_path(None), Some(case_1));
+        }
+
+        // Case two: no previous config, XDG var exists.
+        // SAFETY: This is fine, this is just a test, and no other test affects env vars.
+        unsafe {
+            std::env::set_var("XDG_CONFIG_HOME", "/tmp");
+        }
+        let mut case_2 = PathBuf::new();
+        case_2.push("/tmp");
+        case_2.push(DEFAULT_CONFIG_FILE_LOCATION);
+
+        // Skip this test if the file already exists.
+        if !case_2.exists() {
+            assert_eq!(get_config_path(None), Some(case_2));
+        }
+
+        // Case one: old non-XDG exists already, XDG var exists.
+        // let case_3 = case_1;
+        // assert_eq!(get_config_path(None), Some(case_1));
+    }
 }
